{% load i18n %}
<div class="modal fade" id="{% block modal_id %}{% endblock %}" tabindex="-1" role="dialog" aria-labelledby="myModalLabel" aria-hidden="true">
  <div class="modal-dialog">
    <div class="modal-content">
      <div class="modal-header">
        <button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">&times;</span></button>
        <h4 class="modal-title" id="myModalLabel">{% block modal_title %}{% endblock modal_title %}</h4>
      </div>
      <div class="modal-body">
        {% block modal_body %}{% endblock modal_body %}
      </div>
      <div class="modal-footer">
        <button type="button" class="btn btn-default" data-dismiss="modal">{% trans "Close" %}</button>
        {% block modal_buttons %}{% endblock modal_buttons %}
      </div>
    </div>
  </div>
</div>
